"""
    "reversion_compare
    Add compare view to django-reversion for comparing two versions of a reversion model.
"""

__version__ = '0.16.0rc0'
__author__ = 'Jens Diemer <django-reversion-compare@jensdiemer.de>'
